import type { Socket } from "bun";
import { USER_DB, USER_DB_structure } from "./configure";
import SQLite from "./bun.sqlite3";

type SocketData = { userAccessToken: boolean };

export enum actionTypes {
  LOGIN = "login",
  GET = "get",
  INSERT = "insert",
  UPDATE = "update",
  DELETE = "delete",
}

interface dataStructure {
  action: actionTypes;
  username?: string;
  password?: string;
  database?: string;
  table?: string;
  checkCols?: { [key: string]: string };
  getCols?: { [key: string]: string };
}

enum Status {
  success = 0,
  usernamePasswordError = 1,
  NotAuthorized = 2,
}

const UserDB = new SQLite(USER_DB, USER_DB_structure);

export const server = Bun.listen({
  hostname: "localhost",
  port: parseInt(Bun.env.PORT as string) || 3070,
  socket: {
    data(socket, data) {
      ActionHandler(socket, JSON.parse(data.toString()));
    },
    open(socket) {
      socket.write(JSON.stringify({ action: "login" } as dataStructure));
    },
  },
});

async function ActionHandler(socket: Socket<undefined>, data: dataStructure) {
  switch (data.action) {
    case actionTypes.LOGIN:
      const user = UserDB.get(
        "users",
        { username: data.username },
        "*"
      ) as Array<{ username: string; password: string; authorization: string }>;
      if (user.length == 0) return Status.usernamePasswordError;
      else if (
        (await Bun.password.verify(data?.password || "", user[0].password)) ==
        false
      )
        return Status.usernamePasswordError;
      else if (
        user[0].authorization != "*" &&
        (JSON.parse(user[0].authorization) as Array<string>).includes(
          data.database || ""
        ) == false
      )
        return Status;
      return Status.usernamePasswordError;
      break;
    case actionTypes.GET:
    case actionTypes.INSERT:
    case actionTypes.UPDATE:
    case actionTypes.DELETE:
      break;
    default:
  }
}
function returnData(status: Status, data: { [key: string]: string }) {
  return JSON.stringify({ status, data });
}
